# Communication

The communication classes/files have only one dependency for XHR requests: axios

So if you want to build another frontend you can pretty much copy all communication files because they are independent of react, semantic ui, and all the other libs

The communication files are located at `frontend/src/communicationLayer/`

The only other file that is related to communication is [frontend/src/setup.ts](../core/setup.md).

## Setup.ts

The setup file sets up some interceptors

- `for responses`
	- if the response status is 401 it shows the login (resets user data so that the next request requires login)
- if the content-type is json it transforms the response into a json object & catches responses where the return code was != ok and shows an error
	- here the datetime strings from backend are translated into moment objects

## Communication layer

The communication layer is divided into smaller files for specific data. The name of the files tells you what data can be send/received which this sub-layer.

Normally every file in the communication layer has a co-part in the backend e.g. `frontend/src/communicationLayer/pLangs.Layer.ts` has a controller in the backend named `PlangsController.cs`.
If this is not the case then the splitting might be bad and should be reconsidered (e.g. the controller in the backend handles too many different data).

All functions are async and must be awaited (promise).
Because we use redux the actions of redux will trigger the invocation of these functions for us.
Normally redux can't handle promises but with the redux middleware [redux-promise-middleware](../overview/usedFrameworks.md##redux-promise-middleware)
 these gets automatically pending/resolved/rejected we only need to declare a reducer action for these actions

For more information about the redux action & reducer stuff see [redux tutorial](redux.md)


## Communication layer types

Because we want static type checking we use typescript types to validate types

For the communication layer we use the same Type-Names for the backend and frontend (so a backend class have a corresponding typescript interface in the frontend)

There are some naming conventions...

Because there is not always a 1-to-1 mapping between the backend (c#) and frontend (js) types we cannot have exactly the same structures. Also `data from backend to frontend` and `data from frontend to backend` have different structure (also true because of 1. reason).

XXX stand for the name of the data e.g. DoExercise

- `XXXFullBase` is the easiest one because this means that the backend and frontend have exactly the same type, so you can expect the backend to have a type `XXXFullBase` too.
	This also means that for sending/receiving from and to backend the same type is used

- `XXXFromBackend` data that is received from backend
- `XXXForBackend` data that is send to backend
- `XXXBase` is the base type that frontend & backend has in common, so data that is always send/received with every request for this data. normally `XXXFromBackend` and `XXXForBackend` derive from this type
- `XXXFrontendExclusive` is a type only used in the frontend. This is e.g. needed if we add some temporarily data the the js object e.g. the left time string for exercises or
	- if we receive data e.g. exercises that is split into smaller parts (exercise consists out of the exercise data, custom tests, normal tests, submit tests, ...)


[TODO ref backend type names]

## Adding communication layers

We assume you already have implemented the backend controller.

First thing to do is adding the corresponding frontend typescript types for the backend types (see above).

After all types are created, create a file inside `frontend/src/communicationLayer/`.
For the name it is a good idea to choose the same name as the controller but the the suffix `Layer`.

This way you can easily tell which controller belongs to which communicationLayer-file


Here is an example

```ts
import * as genericLayer from './genericLayer'
import ... the needed types ...

const callingName = '[insert name of the file here for logging]'
const controllerPrefix = '[path/to/the/controller/api]'

export async function getOwnCustomProjectOverviews(): Promise<ReadonlyArray<CustomProjectPreviewFullBase>> {
  return genericLayer.get<ReadonlyArray<CustomProjectPreviewFullBase>>(callingName,
    `${controllerPrefix}/all`)
}

export function createOrUpdateCustomProject(customProjectPreview: CustomProjectPreviewFullBase): Promise<CustomProjectPreviewFullBase> {
  return genericLayer.create(callingName,
    `${controllerPrefix}`, customProjectPreview)
}

export function UpdateCustomProject(customProjectPreview: CustomProjectPreviewFullBase): Promise<CustomProjectPreviewFullBase> {
  return genericLayer.update(callingName,
    `${controllerPrefix}`, customProjectPreview)
}

export async function deleteOwnCustomProject(customProjectPreview: CustomProjectPreviewFullBase): Promise<void> {
  return genericLayer.del<void>(callingName,
    `${controllerPrefix}/delete/${customProjectPreview.id}`)
}
```

Normally all layers use the genericLayer to make requests.

`genericLayer` is just a helper function which wraps the requests and checks for errors and handles some logging.

See `frontend/src/communicationLayer/genericLayer.ts` for the possible functions

The defined functions inside the new file can then be used e.g. inside redux actions
