# Redux

The official docs for redux are [here](https://redux.js.org)

## Example

So here is the "basic" structure of redux

```mermaid
graph TD
rootStorage -. data as props .-> comp[component1/view]
comp[component1/view] -- click --> actionClick[click action]
comp[component1/view] -- other event --> actionOther[other event action]
actionClick -- click data --> reducer1
actionOther -- event data --> reducer1
reducer1 -- might modify --> rootStorage
reducer1 --> reducer2
reducer2 -- might modify --> rootStorage
```

To make the example not too big we use only 1 component but they can be as many as you want

As you know from [components](components.md) the props for some components are calculated from the `rootStorage`

The `rootStorage` is just a plain js object like

```ts
{
  doExerciseSite: {
    exercise: {
      id: 10,
      description: {
        ...
      }
    },
    userSolution: {
      id: 2,
      files: [
        {
          displayName: "Main.java",
          content: "import xxx\n\npublic class Main {\n}"
        }
      ]
    },
    editorIsDisplayed: true,
    ...
  },
  userData: {
    id: 1,
    firstName: "janis",
    lastName: "dähne",
    systemRole: {
      id: 1,
      displayName: "admin",
      permissions: [...]
    },
    ...
  },
  ...
}
```

The props e.g. for the doExercise-site component will slice out the doExerciseSite data e.g.

```ts

import {
  setEditorIsDisplayed
} from "../../../state/actions/...";

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    exercise: rootState.doExerciseSite.exercise,
    userSolution: rootState.doExerciseSite.userSolution,
    editorIsDisplayed: rootState.doExerciseSite.editorIsDisplayed
    //maybe others too
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  setEditorIsDisplayed
}, dispatch)

class DoExerciseSite extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div>
        <Button onClick={() => this.props.setEditorIsDisplayed(!this.props.editorIsDisplayed)}></Button>

        <Editor isDisplayed={this.props.editorIsDisplayed}>
          {
            //this is a custom component defined in another file
          }
        </Editor>
      </div>
    )
  }
}

```

The next step is that the user (or a timer or something) does an action e.g. clicks on a button

This will *dispatch* the `setEditorIsDisplayed` action. Actions are functions that return a plain js object e.g. the button click should hide the editor

```ts
//file doExerciseActions.ts
export function setEditorIsDisplayed(isDisplayed: boolean): SET_editorIsDisplayedAction {
  return {
    type: ActionType.SET_editorIsDisplayed
    isDisplayed
  }
}
```

The idea is that the object contains all data that is needed to perform the actual action (where the action is to modify the app state)
A component can *dispatch* the set of actions defined in the `mapDispatchToProps` wrapper function

The actions are then propagated to all reducers one by one

The reducer itself it just a simple function with a switch statement like

```ts
//file doExerciseReducer.ts
export type State = {
  readonly editorIsDisplayed: boolean
  readonly exercise: {...}
  readonly userSolution: {...}
}

export enum ActionType {
  SET_editorIsDisplayed = 'doExerciseSite_SET_editorIsDisplayed',
  RESET = 'doExerciseSite_RESET',
}

export interface ActionBase extends Action {
  readonly type: ActionType
}

export interface SET_editorIsDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_editorIsDisplayed
  readonly isDisplayed: boolean
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  | ResetAction
  SET_textAction

export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.SET_editorIsDisplayedAction:
      return {
        ...state,
        editorIsDisplayed: action.isDisplayed,
      }

    case ActionType.RESET:
      return initial

    default:
      //notExhaustive is our function to check if we checked all cases in AllActions
      notExhaustive(action) //this can only be used if we have >= 2 action types!
      return state
  }
}
```

When the action object is passed to a reducer (along with the state for the reducer, see below) the switch is evaluated and because **every** action has it's own type the reducer can decide if it wants to handle the action or not (if not case matches)

When a case is matched the state is "modified", actually copied with the needed changes, and returned and propagated to the next reducer

You might wonder about all the types...

The type returned from the action *function* is the type inside the reducer (because the reducer knows what data it can handle)

The `ActionType` enum define strings that the actions can use as the `type` property. Normally `ActionType` is defined in its own file because they are a lot

After all reducers passed we get a new storage. Redux then calls the `mapStateToProps` and `mapDispatchToProps` functions for all components where the props are affected by the changed values (in the new storage).

[actually redux don't *calls* the functions... it manages the react component state under the hood and uses the state to trigger a state change which is noticed by react which in turn will re-render the component]


## Details

There might be some questions left...

*How the `RootState` looks like?*

```ts
import {reducer as dashboardSiteReducer, State as DashboardSiteState} from './doExerciseReducer.ts'

export interface RootState {
  readonly dashboardSiteState: DashboardSiteState
}

export default combineReducers<RootState>({
    dashboardSiteState: dashboardSiteReducer,
})
```

Where `dashboardSiteReducer` is the `reduce` function in the reducer file and
`DashboardSiteState` is an alias for the `State` type in the reducer file


*If redux passes downs the *state for the reducer* how this is done?*

Obviously it is not the whole `RootState` because then we could not return

```ts
{
  ...state,
  editorIsDisplayed: action.isDisplayed,
}
```

in the case block

So the *magic* happens with the `combineReducers` call inside the `rootState` file.
This will return a new function that calls all reducers
Because we returns an object we define keys `dashboardSiteState` and the connected reducer `dashboardSiteReducer`
With this information it is easy for redux to slice out the right data from the `rootState` for the reducer... something like

```ts
const reducerState = rootState['dashboardSiteState']
const nextState = reduce(reducerState, action)
rootState['dashboardSiteState'] = nextState
```

where `action` is the action object that was dispatched from the button click

It's also easy for redux to tell if something has changed... remember that we never modify objects but create a copy with modified values
If you compare `reducerState == nextState` this will be true if they are the same objects (same reference) but because we copy objects they never have the same reference [immutability yey ;)]

## Structure

The actions are found at `frontend/src/state/actions/` and
the reducers at `frontend/src/state/reducers/` and
`frontend/src/state/state.ts` actually creates the instance of the `rootStorage`

[TODO maybe move the actions to the reducers??]

## Reducer composition

If the reducer state gets bigger e.g. the doExerciseState is fairly big in reality

It currently feels that we implemented reducer composition somehow wrong [official docs](https://redux.js.org/recipes/structuring-reducers/using-combinereducers)

[we don't use combine reducers...]

Our current approach:

If we have a big *slice* of data e.g. in the doExerciseState the exercise we create an own reducer for this slice, like a normal reducer

In the parent reducer (do exercise site reducer) we then have to import all action types (from the child reducer) and add them to the `AllActions` union type of the parent and write them in the case clauses again... like

```ts
export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {
    case ActionType.SET_doExercise:
    case ActionType.SET_tests:
    case ActionType.SET_customTests:
      return {
        ...state,
        exercise: doExerciseReducer(state.exercise, action) //this is the child reducer
      }
  }
}
```

We call the child reducer and pass only the slice in (`state.exercise`)

[TODO there might be a better solution!]

## Middlewares

`frontend/src/state/state.ts` also defines the used middlewares e.g. loggers

something to automatically resolve promises and chain actions, see sections below

## Async actions

A lot of component (especially the site components) need to perform async actions e.g. querying data from the backend

For this purpose there is a special lib called [redux promise middleware](https://github.com/pburtchaell/redux-promise-middleware)

The async action and reducers are normally located inside the parent reducer folder inside a folder called `crud`


So when you dispatch an async action the action needs to have a

- `type` property (like all actions)
- `payload` property  which is the promise

Normally we wrap the payload inside the `delay` debug function [actually we don't know why we left it there, it was done to *see* if the loading animations work properly]

The promise function itself is a function from the [communication layer](../core/communication.md) to get data from the backend

After the action object is created the middleware will do the following for us (assume the `type` prop of the action obj was set to `oldTypeName`)

- dispatch a new action with the type set to `oldTypeName_PENDING`, can be used to show loading animations
- if the promise was fulfilled
  - dispatch a new action with type set to `oldTypeName_FULFILLED`
- if the promise was rejected
  dispatch a new action with type set to `oldTypeName_FULFILLED`


For async actions use the `ReduxPromiseAction` which should help you to get the types right
The usage is not that easy... it might be the best to copy an existing reducer and change the types and data as you need it

*some hints*

- `ReduxPromiseAction` change 1. type to the type you expect from the request
- `ReduxPromiseFulfilledAction` change 1. to the type you expect from the request
- `ReduxPromiseRejectedAction` 1. type is normally void because if the request is rejected no data is returned (normally)

## Multiple actions

At some point single actions (returning just an object) is not a good solution because of all the boilerplate code

Luckily there is another middleware called [redux thunk](https://github.com/gaearon/redux-thunk) which can be used to do just this

We create a special type for this called `MultiActions`, usage

```ts
export function setMultipleThings(someData: SomeData): MultiActions {
  return (dispatch, getState) => {

    dispatch(setEmail(someData.email))

    dispatch(setLangId(someData.langId))

    dispatch(setIsDisplayed(!getState().someDataState.isDisplayed))
  }
}
```

Which will dispatch all actions to the reducers
You can even use `getState()` to get the current state

## Multiple async actions

Want to combine multiple and async actions? There is a type for this `AwaitActions`
e.g. for the do exercise site you want to get the exercise and the user solution

This is especially useful for site components when they need to load multiple data

```ts
export function loadDoExerciseSite(releaseCode: string): AwaitActions {
  return async (dispatch, getState) => {

    await dispatch(getDoExerciseAsync(releaseCode))

    await dispatch(getDoExerciseSolution(releaseCode))

    dispatch(setDoExerciseEditorEnabled(true))
  }
}
```


## Why no component states

We don't **ever** use the react component local state... why?

First because thats bad with redux (mentioned in the redux official docs)

A more sophisticated reason is that we have the `rootStorage` as single *source of truth*
We want to be able to reconstruct the whole app state (data & views) with only this `rootState`

Consider the login timeout... if the timeout is hit all components are destroyed (if we only hide them it would be easy for a user to change some css and make them visible again)
If the component is destroyed all local data is lost thus we cannot return to the state before the timeout was hit and after login *all* data would be gone!

With the `rootState` we can rebuild the whole app with only the single js object.

There are other nice side effects (even if they are not used yet) e.g. time travel debugging - because we always copy a state we could save all states (or only the changes via the action objects) and go to every point in time when some button was clicked or some other action performed (because all actions will need to modify the `rootState` in order to do something)


---

TODO maybe add dedicated sections on how to add a full new reducer & actions??